program rotatelogo;
uses crt,dos;
{$M 16384, 0, 32768}


type
  size = array[0..199,0..319] of byte;
  color = array[0..255,1..3] of byte;

Var   SinTable,CosTable: Array[0..255] of integer;
      Sin2Table,Cos2Table: Array[0..255] of integer;
      Map:word; {used as a pointer to the bitmap}
      screen : size absolute $A000:$0000;
      sprite : array[0..1023] of byte;
      key : char;
      fname : string;
      px,py,c : integer;
      fanim : file;
      fanimres : word;
      colors : color;
      fanimcnt : byte;

 procedure cyclecolor(var _color : color;_first,_last : byte);
 var __misc,__r,__g,__b : byte;
 begin
   __r := _color[_last,1];
   __g := _color[_last,2];
   __b := _color[_last,3];
   for __misc := _last downto _first do
   begin
     _color[__misc,1] := _color[__misc-1,1];
     _color[__misc,2] := _color[__misc-1,2];
     _color[__misc,3] := _color[__misc-1,3];
   end;
   _color[_first,1] := __r;
   _color[_first,2] := __g;
   _color[_first,3] := __b;
 end;

 procedure setcolor(_color,_r,_g,_b : byte); assembler;
 asm
   mov dx,$3C8
   mov al,_color
   out dx,al
   inc dx
   mov al,_r
   out dx,al
   mov al,_g
   out dx,al
   mov al,_b
   out dx,al
 end;

 procedure allcolors(_color : color);
 var __misc : byte;
 begin
   for __misc := 0 to 255 do
     setcolor(__misc,_color[__misc,1],_color[__misc,2],_color[__misc,3]);
 end;

 procedure setcolors(_color : color;_first,_last : byte);
 var __misc : byte;
 begin
   for __misc := _first to _last do
     setcolor(__misc,_color[__misc,1],_color[__misc,2],_color[__misc,3]);
 end;


Procedure MakeTables;                   {Creates sin/cos tables}
Var direction:integer;
    angle:real;
begin
     For Direction:=0 to 255 do begin   {use 256 degrees in circle}
         angle:=Direction;
         angle:=angle*3.14159265/128;
         SinTable[Direction]:=round(Sin(angle)*256);
         CosTable[Direction]:=round(Cos(angle)*256);
         Sin2Table[Direction]:=round(Sin(angle+3.14159265/2)*256*1.2);
         Cos2Table[Direction]:=round(Cos(angle+3.14159265/2)*256*1.2);
     end;                 { the 1.2 accounts for pixel aspect ratio }
end;

Procedure DrawScreen(x,y,scale:word; rot:byte);
var Temp:Longint;            {used for intermediate large values}
    ddx,ddy,d2x,d2y:integer;
    i,j:word;
    label hloop,vloop,nodraw;

begin
{ the following 8 lines of code calculate a 'right' and 'down' vector used
  for scanning the source bitmap. I use quotes because these directions
  depend on the rotation. For example, with a rotation, 'right' could mean
  up and to the left while 'down' means up and to the right. Since the
  destination image (screen) is scanned left-right/top-bottom, the bitmap
  needs to be scanned in arbitrary directions to get a rotation. }

     Temp:=(CosTable[rot]);Temp:=(Temp*Scale) div 256;
     ddx:=Temp;
     Temp:=(SinTable[rot]);Temp:=(Temp*Scale) div 256;
     ddy:=Temp;

{ Different tables are used for the 'down' vector to account for the non-
  square pixels in mode 13h (320x200). The 90 degree difference is built
  into the tables. If you don't like that, then use (rot+64)and255 here
  and take the pi/2 out of CreateTables. To each his own I guess. }

     Temp:=(Cos2Table[rot]);Temp:=(Temp*SCALE) div 256;
     d2x:=Temp;
     Temp:=(Sin2Table[rot]);Temp:=(Temp*SCALE) div 256;
     d2y:=Temp;

{ Since we want to rotate around the CENTER of the screen and not the upper
  left corner, we need to move 160 pixels 'left' and 100 'up' in the bitmap.}

     i:=x-ddx*160-d2x*100; j:=y-ddy*160-d2y*100;

{ The following chunk of assembly does the good stuff. It redraws the entire
  screen by scanning left-right/top-bottom on screen while also scanning the
  bitmap in the arbitrary directions determined above. }

         ASM
                 push ds
                 mov  ax,[Map]      {get segment of bitmap}
                 mov  ds,ax
                 mov  ax,$a000      {set es: to video memory}
                 mov  es,ax
                 mov  ax,0          {set ds: to upper left corner of}
                 mov  di,ax         {the video memory}
                 mov  ax,[ddx]      {this is just to speed things up later}
                 mov  si,ax         {add ax,si  faster than  add ax,[ddx] }
                 mov  cx,200        {Number of rows on Screen}
         vloop:
                 push cx
                 mov  ax,[i]        {start scanning the source bitmap}
                 mov  dx,[j]        {at i,j which were calculated above.}
                 mov  cx,320        {Number of coulumns on screen}
         hloop:
                 add  ax,si        {add the 'right' vector to the current}
                 add  dx,[ddy]     {bitmap coordinates.  8.8 fixed point}
                 mov  bl,ah        {  bx = 256*int(y)+int(x)  }
                 mov  bh,dh
                 mov  bl,[ds:bx]   { load a pixel from source }
                 or   bl,bl
                 jz   nodraw
                 mov  [es:di],bl   { copy it to destination }
         nodraw: inc  di           { advance to next destination pixel }

         {*** by repeating the above 7 instructions 5 times, and reducing
              the loop count to 64, I have hit 37fps on a 486-33 with a
              fast video card. ***}

                 loop hloop         {End of horizontal loop}

                 mov  ax,d2x        { get the 'down' vector }
                 mov  dx,d2y

              { add  si,2 }    {** uncomment this instr. for extra fun **}

                 add  i,ax          { i,j is the starting coords for a line }
                 add  j,dx          { so this moves down one line }
                 pop  cx            { get the row count back and loop }
                 loop vloop         { End of verticle loop }
                 pop  ds            { Restore the ds }
         end;

end;

Procedure eraseScreen(x,y,scale:word; rot:byte);
var Temp:Longint;            {used for intermediate large values}
    ddx,ddy,d2x,d2y:integer;
    i,j:word;
    label hloop,vloop,nodraw;

begin
{ the following 8 lines of code calculate a 'right' and 'down' vector used
  for scanning the source bitmap. I use quotes because these directions
  depend on the rotation. For example, with a rotation, 'right' could mean
  up and to the left while 'down' means up and to the right. Since the
  destination image (screen) is scanned left-right/top-bottom, the bitmap
  needs to be scanned in arbitrary directions to get a rotation. }

     Temp:=(CosTable[rot]);Temp:=(Temp*Scale) div 256;
     ddx:=Temp;
     Temp:=(SinTable[rot]);Temp:=(Temp*Scale) div 256;
     ddy:=Temp;

{ Different tables are used for the 'down' vector to account for the non-
  square pixels in mode 13h (320x200). The 90 degree difference is built
  into the tables. If you don't like that, then use (rot+64)and255 here
  and take the pi/2 out of CreateTables. To each his own I guess. }

     Temp:=(Cos2Table[rot]);Temp:=(Temp*SCALE) div 256;
     d2x:=Temp;
     Temp:=(Sin2Table[rot]);Temp:=(Temp*SCALE) div 256;
     d2y:=Temp;

{ Since we want to rotate around the CENTER of the screen and not the upper
  left corner, we need to move 160 pixels 'left' and 100 'up' in the bitmap.}

     i:=x-ddx*160-d2x*100; j:=y-ddy*160-d2y*100;

{ The following chunk of assembly does the good stuff. It redraws the entire
  screen by scanning left-right/top-bottom on screen while also scanning the
  bitmap in the arbitrary directions determined above. }

         ASM
                 push ds
                 mov  ax,[Map]      {get segment of bitmap}
                 mov  ds,ax
                 mov  ax,$a000      {set es: to video memory}
                 mov  es,ax
                 mov  ax,0          {set ds: to upper left corner of}
                 mov  di,ax         {the video memory}
                 mov  ax,[ddx]      {this is just to speed things up later}
                 mov  si,ax         {add ax,si  faster than  add ax,[ddx] }
                 mov  cx,200        {Number of rows on Screen}
         vloop:
                 push cx
                 mov  ax,[i]        {start scanning the source bitmap}
                 mov  dx,[j]        {at i,j which were calculated above.}
                 mov  cx,320        {Number of coulumns on screen}
         hloop:
                 add  ax,si        {add the 'right' vector to the current}
                 add  dx,[ddy]     {bitmap coordinates.  8.8 fixed point}
                 mov  bl,ah        {  bx = 256*int(y)+int(x)  }
                 mov  bh,dh
                 mov  bl,[ds:bx]   { load a pixel from source }
                 or   bl,bl
                 jz   nodraw
                 mov  bl, 0
                 mov  [es:di],bl   { copy it to destination }
         nodraw: inc  di           { advance to next destination pixel }

         {*** by repeating the above 7 instructions 5 times, and reducing
              the loop count to 64, I have hit 37fps on a 486-33 with a
              fast video card. ***}

                 loop hloop         {End of horizontal loop}

                 mov  ax,d2x        { get the 'down' vector }
                 mov  dx,d2y

              { add  si,2 }    {** uncomment this instr. for extra fun **}

                 add  i,ax          { i,j is the starting coords for a line }
                 add  j,dx          { so this moves down one line }
                 pop  cx            { get the row count back and loop }
                 loop vloop         { End of verticle loop }
                 pop  ds            { Restore the ds }
         end;

end;

Procedure AllocateMem;  {returns a segment pointer for a 64K bitmap}
label noerror;
begin
     asm
              mov   ah,$48
              mov   bx,$1000     { request 64K }
              int   $21
              jnc   noerror
              mov   ax,0000
     noerror: mov   Map,ax       { The segment pointer goes in Map }
              end;
     If Map=0 then begin
        Writeln('Could not allocate enough memory');
        Writeln('Program ending...');
        Halt;end;
end;

Procedure GiveBackMem; {returns the memory used for the map to the system}
begin
     asm
        mov  ah,$49
        mov  dx,Map
        mov  es,dx
        int  $21
     end;
end;

Var    rot,dr:word;
       x,y,dist,dd:word;

{procedure setColor(color,r,g,b:byte);assembler;asm
  mov dx,$3C8
  mov al,color
  out dx,al
  inc dx
  mov al,r
  out dx,al
  mov al,g
  out dx,al
  mov al,b
  out dx,al
end;}

const
  spark1x = 100;
  spark1y = 53;
  spark2x = 237;
  spark2y = 70;
  spark3x = 143;
  spark3y = 117;



var
  dat    : file of size;
  p      : file of color;
  done   : boolean;
  spark  : byte;

procedure setall(s : color);
var m235 : integer;
begin
  for m235 := 0 to 255 do
    setcolor(m235,s[m235,1],s[m235,2],s[m235,3]);
end;

procedure fadein(s : color);
var s1 : color; d : boolean; m : integer;
begin
  d:= false;
  fillchar(s1,sizeof(s1),0);
  while d = false do
  begin
    for m := 0 to 255 do
    begin
      if (s1[m,1] < s[m,1]) then inc(s1[m,1]);
      if (s1[m,2] < s[m,2]) then inc(s1[m,2]);
      if (s1[m,3] < s[m,3]) then inc(s1[m,3]);
      d := true;
      if (s1[m,1] < s[m,1]) or
         (s1[m,2] < s[m,2]) or
         (s1[m,3] < s[m,3]) then d := false;
    end;
    delay(50);
    setall(s1);
  end;
end;

procedure fadeout(s : color);
var s1 : color; d : boolean; m : integer;
begin
  d:= false;
  move(s,s1,768);
  while d = false do
  begin
    for m := 0 to 255 do
    begin
      if (s1[m,1] > 0) then dec(s1[m,1]);
      if (s1[m,2] > 0) then dec(s1[m,2]);
      if (s1[m,3] > 0) then dec(s1[m,3]);
      d := true;
      if (s1[m,1] > 0) or
         (s1[m,2] > 0) or
         (s1[m,3] > 0) then d := false;
    end;
    delay(50);
    setall(s1);
  end;
end;

procedure sparkle(x,y : integer);
var misc : integer;  bitmap : array[0..100] of byte;
begin
  for misc := 0 to 2 do
  begin
    bitmap[misc] := screen[y-misc,x-misc];
    bitmap[(misc+10)] := screen[y+misc,x+misc];
    bitmap[(misc+20)] := screen[y-misc,x+misc];
    bitmap[(misc+30)] := screen[y+misc,x-misc];
    screen[y-misc,x-misc] := 7-misc;
    screen[y+misc,x+misc] := 7-misc;
    screen[y-misc,x+misc] := 7-misc;
    screen[y+misc,x-misc] := 7-misc;
    delay(50);
  end;
  bitmap[69] := screen[y-3,x+3];
  screen[y-3,x+3] := 7-3;
  for misc := 3 to 6 do
  begin
    bitmap[misc+40] := screen[y-misc,x-misc];
    screen[y-misc,x-misc] := 7-misc;
    delay(50);
  end;
  delay(100);
  {hide}
  for misc := 6 downto 3 do
  begin
    screen[y-misc,x-misc] := bitmap[misc+40];
    delay(50);
  end;
  screen[y-3,x+3] := bitmap[69];
  for misc := 0 to 2 do
  begin
    screen[y-misc,x-misc] := bitmap[misc];
    screen[y+misc,x+misc] := bitmap[(misc+10)];
    screen[y-misc,x+misc] := bitmap[(misc+20)];
    screen[y+misc,x-misc] := bitmap[(misc+30)];
    delay(50);
  end;
end;

var
 speed : integer;

begin
  if paramcount < 1 then fname := 'picrain' else fname := paramstr(1);
  AllocateMem;
  asm
    mov ax,13h
    int 10h
  end;
  fillchar(colors,sizeof(colors),0);
  allcolors(colors);
  assign(dat,fname+'.dat');
  reset(dat);
  read(dat,screen);
  close(dat);
  assign(p,fname+'.pal');
  reset(p);
  read(p,colors);
  close(p);          { 0..7 black-white grey scale }
  done := false;
  for y := 1 to 200 do
    move(screen[y-1,74],mem[map:(y*256)],256);
  spark := 0;
  x:=32768; y:=0;         {this corresponds to (128,0) in fixed point}
  rot:=0; dr:=1;          {rotation angle and it's delta}
  dist:=200; dd:=65534; {distance to bitmap (sort of) and its delta}
  fillchar(screen,64000,0);
  for y := 200 to 256 do
    move(screen[y-200],mem[map:(y*256)],256);
  MakeTables;
{  assign(fanim,'ranim.dat');
  rewrite(fanim);}
  fadein(colors);
  fanimcnt := 0;
  key := #1;
  repeat
{   inc(spark);
    if spark < 4 then
    if spark = 1 then sparkle(spark1x,spark1y);
    if spark = 2 then sparkle(spark2x,spark2y);
    if spark = 3 then sparkle(spark3x,spark3y);      }
        DrawScreen(x,y,dist,lo(rot));
{       delay(50);
        erasescreen(x,y,dist,lo(rot)); }
        rot:=rot+dr;
        y:=y+1280;      {slow panning. 1/2 pixel per frame}
        dist:=(dist+dd);
        if (dist=1000) or (dist=2) then dd:=-dd;
        if random(150)=1 then dr:=random(7)-3;
    key := #1;
    cyclecolor(colors,1,255);
    allcolors(colors);
{    px := random(255);
    py := random(255);
    c := px xor py;
    mem[map:py*256+px] := c;}
    if (keypressed) then
    begin
      key := readkey;
{      if key = '`' then
        inc(fanimcnt);
      if key = ' ' then}
        done := true;
    end;
    if (fanimcnt > 0) and (fanimcnt < 26) then
    begin
      blockwrite(fanim,screen,500,fanimres);
      inc(fanimcnt);
    end;
  until done = true;
{  close(fanim);   }
  GiveBackMem;
  fadeout(colors);
  asm
    mov ax, 3h
    int 10h
  end;
  writeln('Rainbow Swirl 1995 by Rick Tilley');
  writeln('I hope you enjoyed this program.  You can reach me at:');
  writeln('ptilley@haywire.csuhayward.edu');
  writeln('http://haywire.csuhayward.edu/~ptilley');
  writeln;
end.